import com.bluedust.math.BigFraction;

/**
 * A simple class to verify the workings of the BigFraction.
 * 
 * @author steev
 *
 */
public class BigFractionTester {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		BigFraction half = new BigFraction(1,2);
		BigFraction third = new BigFraction(1,3);
		BigFraction minusQuarter = new BigFraction(1,-4);
		BigFraction topHeavy = new BigFraction(10,3);
		BigFraction minsTopHeavy = new BigFraction(-10,3);
		
		System.out.println("1/2 * 1/3 = "+BigFraction.toString(half.multiply(third)));
		System.out.println("1/2 / 1/3 = "+BigFraction.toString(half.divide(third)));
		System.out.println("1/2 + 1/3 = "+BigFraction.toString(half.add(third)));
		System.out.println("1/2 - 1/3 = "+BigFraction.toString(half.subtract(third)));
		System.out.println("1/(/2) = "+BigFraction.toString(half.reciprocal()));
		System.out.println("-1/2 = "+BigFraction.toString(half.negate()));
		System.out.println("-1/4 = "+BigFraction.toString(minusQuarter));
		System.out.println("--1/4 = "+BigFraction.toString(minusQuarter.abs()));
		System.out.println("1/2 < 1/3 = "+((half.compareTo(third)==-1)?"true":"false"));
		System.out.println("-1/4 < 1/3 = "+((minusQuarter.compareTo(third)==-1)?"true":"false"));
		System.out.println("int(-1/4) = "+minusQuarter.intValue());
		System.out.println("float(-1/4) = "+minusQuarter.floatValue());

		System.out.println("10/3 = "+BigFraction.toString(topHeavy));
		System.out.println("-10/3 = "+BigFraction.toString(minsTopHeavy));
		System.out.println("-10/3 * 10/3 = "+BigFraction.toString(minsTopHeavy.multiply(topHeavy)));

		// Calculating pi is not a real test of fractional numbers, since it is
		// irrational. But we'll do it anyway!
		// p.s. these series many hundreds of terms to converge on the first few 
		// decimal places, so the answers given here will appear wrong.
		// Also note that the fractional components are larger than the range capable
		// of being represented by floats, and therefore floatValue will return NaN
		// if the iteration count is increased by much.
		
		
		// A *really* bad way of find pi, through the Gregory-Leibniz series
		// Basically it's, 4/1 - 4/3 + 4/5 - 4/7 + 4/9 etc
		BigFraction pi = new BigFraction(0);
		int denom = 1;
		for(int i=0;i<21;++i) {
			BigFraction term = new BigFraction(4,denom);
			if ((i&1)==1) {
				pi = pi.subtract(term);
			} else {
				pi = pi.add(term);
			}
			denom += 2;
		}
		System.out.println("pi ~= "+BigFraction.toString(pi));
		System.out.println("float(pi) ~= "+pi.floatValue());
		
		// A slightly better method using Wallis's product.
		// pi/2 = 2/1 * 2/3 * 4/3 * 4/5 * 6/5 * 6/7
		BigFraction piWallis = new BigFraction(1);
		int nom = 2;
		denom = 1;

		for(int i=0;i<60;++i) {
			piWallis = piWallis.multiply(new BigFraction(nom, denom));
			if ((i&1)==1) {
				nom += 2;
			} else {
				denom += 2;
			}
		}
		piWallis = piWallis.multiply(new BigFraction(2));
		System.out.println("pi ~= "+BigFraction.toString(piWallis));
		System.out.println("float(pi) ~= "+piWallis.floatValue());
		
		
		
	}

}